	page    132,60
	opt	CEX,MEX,PP
;----------------------------------------------------------------------------
; BPSK.ASM  
;
; Experimental DSP interface program for the VE2IQ COHERENT program.
; Copyright (C) Johan Forrer, KC7WW, 1997. 
;
; Depending on compile directive, this code will generate C-BPSK with
; a) Unshaped pulses (mild FIR filtering) SHAPING=OFF.
; b) PSK31-style waves SHAPING=ON, BLOB=OFF. 
; c) "BLOB" waves SHAPING=ON, BLOB=ON. 
; The default is set for BLOB waves.
; When SHAPING=ON, one can further select the kind of shaping functions.
; The default is Cosine.
;
; NOTE:
; Pulse shaping works only for MS25! Change the symbol length if needed.
; DTR timeout is set for 100mS, for large MS values this may 
; not be sufficient
;
;----------------------------------------------------------------------------
; Obtain COHERENT from http://www.ietc.ca/home/bill/bbs.htm
;
; A couple of important points:
;----------------------------------------------------------------------------
; 1) If you are planning on using the TAPR interface kit, you will need
;    to make a couple of small changes to the program. This code presently
;    uses PB8 and PB9 as inputs, the TAPR interface is using PB9 and PB10.
; 2) Change the EVM's ceramic resonator for a 4 MHz crystal.
; 3) Wire in a pair of RS232 to TTL convertors to change the
;    signal levels of RTS (used by COHERENT as the BPSK signal),
;    and DTR (used by COHERENT as the PTT switch). These level
;    shifters can be someting really simple. I connect the RS232
;    line in question to a 10K series resistor feeding into the
;    base of a 2N2222 transistor. The base of the transistor is 
;    then protected by a 1N914 diode (cathode to the transistor
;    base, anode to ground). The transistor emitter is grounded,
;    and the collector is then taken via a 1K resistor to +5V. 
;    I then routed a wire from the collector of each transistor
;    to PB8 and PB9 corresponding to RTS and DTR respectively.
; 4) Notice that the RED LED is your overload indicator - it has been
;    arranged that overload at the EVM corresponds to the CLIPPING
;    level for COHERENT.
; 5) The GREEN LED flashes at 1 second intervals. Check the accuracy
;    of your system against the second ticks on WWV. It should hold
;    time even for extended periods, however, if you can hold time for
;    even a minute, you are marginal, but it will work in a fashion.
; 6) The rest is up to you: study the COHERENT documentation for how
;    to use the program.
;
;    Good Luck!
;----------------------------------------------------------------------------
; Copyright (C) Johan Forrer, 1997, 
;----------------------------------------------------------------------------
; Johan B. Forrer, KC7WW
; 26553 Priceview Drive, Monroe, OR 97456, USA 
; <forrerj@peak.org>
; July, 1997.
; Revised for BLOB waveform November 1997
;
; Disclaimer:
; This program is provided "as is" and is intended for educational
; purposes only. The author(s) cannot be held resonsible for any 
; mishap or accident resulting through the use of this software.
;  --Use at your own risk--
;
; This software is not to be used for any commercial application,
; military or government agencies without explicit written approval of the
; author(s).
;
; Acknowledgements:
; Pawel Jalocha, SP9VRC for many pieces of this code.
; Bill DeClarle, VE2IQ for help and assistance.
;
; MUCH-NEEDED IMPROVEMENTS:
; The sample rate changer introduces aliasing - fortunately the third order
; products are away from the 800 Hz passband, so that is why things
; works. If anyone adds a nice polyphase rate M/N rate changer, please keep
; me posted.
;----------------------------------------------------------------------------

	include 'sincod.asm'

;----------------------------------------------------------------------------
TONE_OUTPUT     EQU     HEADPHONE_EN+LINEOUT_EN+(0*LEFT_ATTN)+(0*RIGHT_ATTN)   
TONE_INPUT      EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)
OVR_CLR         EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)+(10*RIGHT_GAIN)+(10*LEFT_GAIN)+$200000
;----------------------------------------------------------------------------
; Macros
;----------------------------------------------------------------------------
GrnLED  macro mode      ; Green LED clr/set/chg
	b\mode	#14,x:PBD
        endm

RedLED	macro	mode	; Red   LED clr/set/chg
	b\mode	#13,x:PBD
	endm

LeftPTT macro mode      ;PTT line: clr/set/chg
        b\mode #0,x:PBD
;       b\mode #4,x:PBD
        endm

RightPTT macro mode      ;PTT line: clr/set/chg
        b\mode #4,x:PBD
;       b\mode #0,x:PBD
        endm
;----------------------------------------------------------------------------

Carrier  	equ 	800.0
SampleFreq 	equ 	9600.0
;PassFilterWidth equ	320.0  	; Input  bandwidth
PassFilterWidth equ	100.0  	; Input  bandwidth
OutFilterWidth 	equ  	320.0  	; Output bandwidth

PassFilterLen    equ 	64
OutFilterLen	 equ    64

PTT_TO	equ	@cvi(SampleFreq) ; 1 second on PTT timeout

;----------------------------------------------------------------------------
LEFT 	equ	1
RIGHT	equ	2
usesig	equ	LEFT		; ***Use LEFT audio input***

ON	equ	1
OFF	equ	0

;------------------Pulse shaping----------------------------------------------
; The input bandpass filter can be selected to accomodate squarish input
; pulses, or alternately, shaped raised cosine pulses. This must specified.
;
; Output pulse shaping, or non shaping, can be specified. If it is
; specified, filtering is not needed. There is a choice as to the shaping
; method. That may be either BLOB or the older PSK31-style shaping. Further,
; chice is in the actual ramp shape: Cosine is a litle smoother and musical 
; than the Blackman-Harris. Most folks use Cosine shaping, the Blackman-
; Harris gives a more compact spectrum.
;-----------------------------------------------------------------------------

REGULAR	equ	OFF		; These control in input bandpass filter
G3PLX	equ	ON 		; select only one

SHAPING	equ	ON		; For pulse shaping=ON, filtering only=OFF
				; These select the type of shaping required
BLOB	equ	ON		; Blob or PSK31/style (OFF)
COSINE 	equ	ON		; Raised Cosine
BH	equ	OFF		; Blackman Harris

 IF (SHAPING==ON)
OUTFILTER equ	OFF		; For output filtering
 ELSE
OUTFILTER equ	ON
 ENDIF

;-----------------------------------------------------------------------------

	org     p:
main

; Initialize SC0, SC1 as SCI pins
    	bset    #0,x:PCC                 ; RXD
    	bset    #1,x:PCC                 ; TXD

; initialize port B
	movep	    #$0000,x:PBC	    ; port B as general purpose port
	movep	    #$60ff,x:PBDDR	    ; PB0-PB7,PB13 and PB14 as outputs
	movep	    #$0000,x:PBD


	movep	#$1,x:TCSR		; timer mode 0, GPIO
;;	movep	#>1249,x:TCR		; 36Mhz/2/7200/2 - 1  double rate for tests
	movep	#>2499,x:TCR		; 36Mhz/2/7200/2 - 1  timer factor

	move	#>3600,a
	move	a,x:tcount		; clear tcount


; Initialize SCI port
; Use internal clocks, 115.2 bps
;	
	movep	#>$0302,x:SCR		; 1 start, 1 stop, 8 data (10 bits) 

	movep	#>$0004,x:SCCR		; CD=(36 MHz/bps/64) - 1 = 4 (115.2)
					; (112500 or -2.34%)
;;	movep	#>$003A,x:SCCR		; CD=(36 MHz/bps/64) - 1 = 58 (9.6)
;;					; (9533.9 or -0.69%)


	movep	#$02301b,x:IPR		; IRQA/IRQB/SSI are level 3 interrupts.
					; and are LEVEL sensitive.
					; Timer is level 1 (SCI are polled).
	movep   #$fb00,x:CRB            ; Enable the SSI interrupts.

	movep	#>$3,x:TCSR		; Enable Timer interrupts	

	
        ori #%00000100,omr      	; enable the sine ROM table
                                	; this is for the FreqSynth routine

;
; Program headphone,lineout enabled, speaker off, attn=0
; Does this need to done on evey cycle or once in the beginning?
;
	move    #TONE_OUTPUT,y0         ; headphones, line out, 
					; mute spkr, no attn.
	move    y0,x:TX_BUFF_BASE+2
	move    #TONE_INPUT,y0          ; no input gain, monitor mute
	move    y0,x:TX_BUFF_BASE+3


;---------------------------------------------------------------------------
; Initialize buffer pointer
;---------------------------------------------------------------------------
	move	#Buffer,r1	; data_in_ptr = input buffer pointer
	move	r1,x:data_in_ptr

	move	#OutBuf,r1	; out_ptr = output buffer pointer
	move	r1,x:out_ptr

	move	#TxSymbolShape,r1 	; beginning of array
	move	r1,x:shape_ptr
	
	clr	a	    		; initial phase state
	move	a,x:polarity
	move	a,x:Sym_cnt	
	move	a,x:failsafe		; disable PTT
	move	a,x:timeout		; indicate a timeout condition

;---------------------------------------------------------------------------
; Main Loop
loop_1 
	jset    #2,x:SSISR,*            ; Wait for frame sync to pass.
	jclr    #2,x:SSISR,*            ; Wait for frame sync.

	move    x:RX_BUFF_BASE,a        ; Load the left channel input.
	move    x:RX_BUFF_BASE+1,b      ; Load the right channel input.
	

;----------------------------------------------------------------------------
; Read input signal.
; Note: Input may be overdriven, which case results will saturate.
; Detect overload condition from Slot 7, bit 5 (RX_BUFF_BASE+3 = $20)
; or when read left justified into a 24-bit 20xxxx.
; The bit is reset by writing zero.
;----------------------------------------------------------------------------
	move	a1,x:LEFT_sig		; save A/D data
	move	b1,x:RIGHT_sig

;----------------------------------------------------------------------------
; Read status of switches/RTS/DTR
;----------------------------------------------------------------------------
	movep	x:PBD,a
	move	a1,x:switches

;----------------------------------------------------------------------------
; Test for CODEC overload condition
;----------------------------------------------------------------------------
	move    x:<RX_BUFF_BASE+3,a
	move	#>$200000,x0
	and	x0,a
	jeq	sig_good
	RedLED	set
	move    #TONE_INPUT,y0		; clear overload bit
	move    y0,x:<TX_BUFF_BASE+3
	jmp	process
sig_good
	RedLED	clr
	move    #OVR_CLR,y0		; enable overload detection
	move    y0,x:<TX_BUFF_BASE+3
process

;----------------------------------------------------------------------------
; To minimize audio leakage fouling up measurements,
; only output the 800 Hz carrier when PTT is asserted.
;----------------------------------------------------------------------------
	move	x:switches,a
	move	#>$200,x0
	and	x0,a1
	jne	Receive

;-----------------------------------------------------------------------------
; PTT is asserted, output tone to left and right channel
;-----------------------------------------------------------------------------
	btst	#0,x:failsafe		; test failsafe
	jcc	Receive

	move	x:Toneout,a
	asr	a
	move    a1,x:TX_BUFF_BASE
	move    a1,x:TX_BUFF_BASE+1
	jmp	Setptrs	

;----------------------------------------------------------------------------
; Play back audio for testing purposes
;----------------------------------------------------------------------------
Receive
	move	x:Bpfout,a		; Retrieve data
	rep #8
	asl	a
	move    a1,x:TX_BUFF_BASE        ; Put value in left channel tx.
	move    a1,x:TX_BUFF_BASE+1      ; Put value in right channel tx.  

;----------------------------------------------------------------------------
; Prepare data pointers to receive sampled data
;----------------------------------------------------------------------------
Setptrs
	move	x:data_in_ptr,r1
        move    #PassFilterLen-1,m1     ; mod(ntaps)

 IF (usesig==LEFT) 			; Select which input to use
	move	x:LEFT_sig,a		; data appears left justified
	move	a1,x0
	mpy	x0,#8,a
	move	a1,x:(r1)+		; save in circular buffer
 ENDIF

 IF (usesig==RIGHT)
	move	x:RIGHT_sig,a		; data appears left justified
	move	a1,x0
	mpy	x0,#8,a
	move	a1,x:(r1)+		; save in circular buffer
 ENDIF

	move	r1,x:data_in_ptr	; save updated pointer

;---------------------------------------------------------------------------
; Perform input bandpass filtering
; Sample rate = 9600 SPS
; On completion, pointers is same as in beginning, accum a contains result
;---------------------------------------------------------------------------
        move    #PassFilter,r4	; R4 point to filter coefficients
        move    #>-1,m4     	; linear addressing
	nop

        clr     a    x:(r1)+,x0  y:(r4)+,y0     ; priming registers
        .loop	#PassFilterLen-1		; allow interrupts
	mac     x0,y0,a  x:(r1)+,x0  y:(r4)+,y0
	.endl
        macr    x0,y0,a		; final rounding
	move	a,x:Bpfout	; save it


;-----------------------------------------------------------------------------
; Either pass the sinusoid unchanged or invert it
;-----------------------------------------------------------------------------
	jset	#0,x:polarity,passthru
	move	x:cosine,a
	neg	a
	jmp	output

passthru
	move	x:cosine,a
output			

 

 IF (SHAPING==ON)
;-----------------------------------------------------------------------------
; When SHAPING is ON, there is a choice between BLOB or PSK31 style shaping
;-----------------------------------------------------------------------------

 IF (BLOB==ON)
;-----------------------------------------------------------------------------
; If BLOB format selected, shape each and every symbol regardless
;-----------------------------------------------------------------------------
	move	x:switches,b		; get DTR
	move	#>$100,x0
	and	x0,b1

	move	#>1,b			; set BPSK polarity
	move	x0,x:<polarity
	jeq	is_low	
	move	#>0,b
is_low
	move	b,x:<polarity	
	
	move	x:<oldstate,x1		; trap DTR change of state
	cmp	x1,b			; current RTS state is in b
	move	b1,x:<oldstate		; save state for next time
	jeq	loopout

					; DTR changed, resync
	move	#TxSymbolShape,r1 	; beginning of array
	move	r1,x:shape_ptr
	move	#>SymbolShapeLen,b
	move	b,x:Sym_cnt 		; count down a symbol

	move	#>$FF,b	     		; enable failsafe
	move	b1,x:failsafe
	move	#>PTT_TO,b   		; reset timeout counter
	move	b1,x:timeout

loopout
	move	x:Sym_cnt,b		; are we busy up or down ramping?
	tst	b
	jne	keep_ramping	
		     			; no, reset ramping in case no state change
	move	#TxSymbolShape,r1 	; beginning of array
	move	r1,x:shape_ptr
	move	#>SymbolShapeLen,b
	move	b,x:Sym_cnt 		; count down a symbol

keep_ramping
	move	#>1,x0	   		; yes, keep ramping 
	sub	x0,b
	move	b,x:Sym_cnt

	move	x:shape_ptr,r1
        move    #SymbolShapeLen-1,m1     ; mod(ntaps)
	nop
	move	a,y0 
	move	y:(r1)+,x1		; apply shaping
	mpy	x1,y0,a

	move	r1,x:shape_ptr		; save pointer

	move	x:timeout,b		; timeout if RTS did not change
	tst	b
	jeq	_TO
	move	#>1,x0
	sub	 x0,b
	move	b1,x:timeout
	jmp	_lout
_TO		
	clr	b
	move	b,x:failsafe
_lout
 endif		; BLOB==ON
;-----------------------------------------------------------------------------

 IF (BLOB==OFF)
;-----------------------------------------------------------------------------
; If BLOB not selected, use the older pulse shape method
; The algorithm works as follow:
;  if DTR changes state, start a ramping down of the amplitude.
;      when the amplitude is down to zero,
;                        change the polarity of the sinusoid,
;                        start ramping up the amplitude.
;  if DTR does not change state and we are not in the process of ramping
;     keep outputting the full amplitude value at the same polarity.
;-----------------------------------------------------------------------------
	move	x:switches,b		; get DTR
	move	#>$100,x0
	and	x0,b1

	move	x:<oldstate,x1
	cmp	x1,b			; current RTS state is in b
	move	b1,x:<oldstate		; save state for next time
	jeq	lupout

					; If DTR changed
					; reset shape table index
	move	#TxSymbolShape+SymbolShapeLen/2,r1 ; to middle of array
	move	r1,x:shape_ptr
	move	#>SymbolShapeLen,b
	move	b,x:Sym_cnt 		; count down a symbol

	move	#>$FF,b	     		; enable failsafe
	move	b1,x:failsafe
	move	#>PTT_TO,b   		; reset timeout counter
	move	b1,x:timeout

lupout
	move	x:Sym_cnt,b		; are we busy up or down ramping?
	tst	b
	jeq	h_stdy	

	move	#>1,x0	   		; yes, keep ramping 
	sub	x0,b
	move	b,x:Sym_cnt

	move	#>SymbolShapeLen/2,x0	; time to invert the carrier?
	cmp	x0,b
	jne	kp_ph
	
	move	#>1,x0			; yes, we only monitor the LSBit
	move	x:polarity,b
	add	x0,b
	move	b,x:polarity

kp_ph
	move	x:shape_ptr,r1
        move    #SymbolShapeLen-1,m1     ; mod(ntaps)
	nop
	move	a,y0 
	move	y:(r1)+,x1		; apply shaping
	mpy	x1,y0,a

	move	r1,x:shape_ptr		; save pointer
	jmp	outfil

h_stdy
	move	x:shape_ptr,r1		; hold steady at last amplitude
	move	a,y0 
	move	y:(r1),x1		; apply shaping
	mpy	x1,y0,a

outfil
	move	x:timeout,b		; timeout if RTS did not change
	tst	b
	jeq	_TO_1
	move	#>1,x0
	sub	 x0,b
	move	b1,x:timeout
	jmp	_lut
_TO_1		
	clr	b
	move	b,x:failsafe
_lut
 endif ; BLOB==OFF

 endif 	; SHAPING
;-----------------------------------------------------------------------------
; If we do not shape pulses, then directly trigger phase reversals.
;-----------------------------------------------------------------------------
 IF (SHAPING==OFF)
	move	x:switches,b		; get DTR
	move	#>$100,x0
	and	x0,b1

	move	x:<oldstate,x1
	cmp	x1,b			; current RTS state is in b
	move	b1,x:<oldstate		; save state for next time
	jeq	loopout
					; If DTR changed
	move	#>1,x0			; yes, we only monitor the LSBit
	move	x:polarity,b
	add	x0,b
	move	b,x:polarity

	move	#>$FF,b	     		; enable failsafe
	move	b1,x:failsafe
	move	#>PTT_TO,b   		; reset timeout counter
	move	b1,x:timeout

loopout
	move	x:timeout,b		; timeout if RTS did not change
	tst	b
	jeq	_TO
	move	#>1,x0
	sub	 x0,b
	move	b1,x:timeout
	jmp	_lout
_TO		
	clr	b
	move	b,x:failsafe
_lout

 endif
;----------------------------------------------------------------------------


 IF (OUTFILTER==ON)
;----------------------------------------------------------------------------
; If selected, prepare to bandpass filter output tone.
;----------------------------------------------------------------------------

	move	x:out_ptr,r1
        move    #PassFilterLen-1,m1     ; mod(ntaps)
	nop
	move	a1,x:(r1)+
	move	r1,x:out_ptr

        move    #OutFilter,r4	; R4 point to filter coefficients
        move    #>-1,m4     	; linear addressing
	nop

        clr     a    x:(r1)+,x0  y:(r4)+,y0     ; priming registers
        .loop	#PassFilterLen-1		; allow interrupts
	mac     x0,y0,a  x:(r1)+,x0  y:(r4)+,y0
	.endl
        macr    x0,y0,a		; final rounding
 endif
;----------------------------------------------------------------------------

	move	a,x:Toneout		; save output tone


;----------------------------------------------------------------------------
; Update carrier phase
;----------------------------------------------------------------------------
        move 	X:<CarPhase,x0  	; get phase
        move 	X:<CarFreq,a    	; advance the phase
        add 	x0,a
        move 	a1,X:<CarPhase
        jsr 	<FreqSynth              ; Compute sine/cosines
					; modifies a,b,x,y,r1,m1,n1
	move	a1,x:cosine
	move	b1,x:sine

;----------------------------------------------------------------------------
; Read and extract DTR status for PTT operation
;----------------------------------------------------------------------------
	move	x:switches,a
	move	#>$200,x0
	and	x0,a1
	jne	Release

	btst	#0,x:failsafe		; test failsafe
	jcc	Release

 IF (usesig==LEFT) 			; Select Left/Right PTT
	LeftPTT set
 else
	RightPTT set
 endif
	jmp	loop_exit

Release
 IF (usesig==LEFT)
	LeftPTT clr
 else
	RightPTT clr
 endif

loop_exit

	jmp     loop_1                  ; Loop back.
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
; Routine computes a cosine/sine pair using the sine ROM
; with a second order (linear+quadrature) approximation 
; between table points.
; 	x,y,r1,m1,n1 are modified.
; 	Maximum error is about 0.7E-6
; 	Execution time 4+64+4 clock cycles including "jsr <IQ_lev2" and "rts"
;-----------------------------------------------------------------------------
PI      equ     3.14159265358979323846
FreqSynth                             	; x0 = angle ( -1 = -PI, +1 = +PI)
        move 	#>$80,x1   		; shift out 8 most significant bits
        mpy 	x0,x1,a  #>$FF,x0
        move 	x0,m1
        and 	x0,a     #>$100,x0
        or 	x0,a     #<$40,n1
        move 	a1,r1      		; put the 8 most significant bits 
					; into r1 with offset = $100
        move 	a0,y0      		; save the remaining bits in y0
        jclr 	#23,y0,SinTable_lev2
        move 	(r1)+
SinTable_lev2
        move 	Y:(r1+n1),x0       	; x0 = coarse cosine
        move 	Y:(r1),x1          	; x1 = coarse sine
        mpyr 	x1,y0,a  #PI/256.0,y1
        tfr 	x0,a  a,x1
        macr 	-x1,y1,a           	;a = fine cosine
        mpyr 	x0,y0,b  Y:(r1),x1
        tfr 	x1,b  b,x1
        macr 	x1,y1,b  #PI*PI/2.0/65536.0,y1  ;b = fine sine
        mpyr 	y0,y0,a  a,x0
        move 	a,y0
        mpyr 	y0,y1,a
        tfr 	x0,a  a,y1
        macr 	-x0,y1,a  b,x1     	;a = super fine cosine
        macr 	-x1,y1,b           	;b = super fine sine
        rts


;*************************************************************************************
; Timer ISR
TIMER:
	move    a0,x:save_a0
	move    a1,x:save_a1
	move    a2,x:save_a2
	move    x0,x:save_x0

	move	x:tcount,a
	move	#>1,x0
	sub	x0,a
	jne	done		
	GrnLED	chg
	move	#>3600,a
done
	move	a1,x:tcount

;----------------------------------------------------------------------------
; Form 7 bits+sign to simulate S-D board
; Data = (Current - Previous), scaled to signed 8-bit value
;----------------------------------------------------------------------------
	move	x:Bpfout,a
	move	x:Previous,x0
	move	a,x:Previous
	sub	x0,a
	rep #7
	asr	a
	move	a,x:temp		; save it

	abs	a			; value >127   
	move	#>127,x0
	sub	x0,a
	jgt	error			; flag error if it does
	move	x:temp,a   		; otherwise its OK
	jmp	outit		
error
	move	x:temp,a
	move	#>$7F,x0
	or	x0,a	
outit
	movep	a1,x:STX

;   					; TEST !!!
;----------------------------------------------------------------------------
; Check whether we received further data from Coherent for TX
;----------------------------------------------------------------------------
;	movep	x:SSR,a1
;	move	#>$04,x0		; RDRF?
;	and	x0,a
;	jeq	nodata
;      	movep	x:SRX,a1
;	move	#>127,x0
;	sub	x0,a
;	asl	a
;	asl	a
;	asl	a
;	asl	a
;	asl	a
;	asl	a
;	asl	a
;	asl	a
;	asl	a
;	move	a,x:Toneout
;nodata:

	move    x:save_a0,a0
	move    x:save_a1,a1
	move    x:save_a2,a2
	move    x:save_x0,x0
	rti

;*************************************************************************************


;-----------------------------------------------------------------------------
       org     x:      
LEFT_sig        ds      1       ;storage for Left Signal
RIGHT_sig       ds      1       ;storage for Left Signal


CarPhase  	dc 	0
CarFreq   	dc 	2.0*Carrier/SampleFreq
sine		ds	1	; Sine/Cosine results
cosine		ds	1

tcount		ds	1	; 7200 Hz downcounter
Previous	ds	1
data_in_ptr	ds	1	; buffer pointer
out_ptr		ds	1	; buffer pointer
shape_ptr	ds	1	; shape table pointer
Bpfout		ds	1	; Raw filtered outputs
Toneout		ds	1	; Carrier tone output
switches	ds	1	; PB inputs
oldstate	ds	1	; previous signaling state
polarity	ds	1	; =0/1 for BPSK
Sym_cnt		ds	1	; used for counting down symbol shaping
failsafe	ds	1	; disable PTT when RTS stops pulsing
timeout		ds	1	; disable PTT when RTS stops pulsing
temp		ds	1	; temporary

save_r0		ds	1 	; Temporary register storage
save_m0		ds	1
save_a0		ds	1
save_a1		ds	1
save_a2		ds	1
save_x0		ds	1

Buffer		dsm	PassFilterLen	; storage for input  data
OutBuf		dsm	PassFilterLen	; storage for output data

;-----------------------------------------------------------------------------

Pi      equ     3.14159265358979323846


h0      equ     0.42      	;Blackman-Harris
h1      equ     0.50		;the transmitted spectrum has no sidelobes.
h2      equ     0.08

p0       equ    0.50             ; G3PLX's DFT factors for main filter
p1       equ    1.15          ;>=64dB at >=31.25 Hz
p2       equ    0.7931
p3       equ    0.1567
p4       equ    0.00049
p5       equ    0.0005



LowFreq = (Carrier-PassFilterWidth/2)/SampleFreq
UppFreq = (Carrier+PassFilterWidth/2)/SampleFreq
OLowFrq = (Carrier-OutFilterWidth/2)/SampleFreq
OUppFrq = (Carrier+OutFilterWidth/2)/SampleFreq
MS	= 25
SymbolShapeLen = @cvi(SampleFreq*MS/1000)

       org     y:      
PassFilter ds PassFilterLen 	; Input filter
OutFilter  ds OutFilterLen 	; Cleanup output filter
       org     y:$1000
TxSymbolShape dsm SymbolShapeLen	; symbol shaping window

	org Y:PassFilter
time = -@cvf(PassFilterLen/2)+0.5
count   set 0
	dup PassFilterLen
angle = Pi*time/@cvf(OutFilterLen)

 IF (REGULAR==ON)
; Regular truncated SIN(X)/X response
Window = 0.75*@cos(angle)+0.25*@cos(3*angle)
Filter = (@sin(2.0*Pi*time*UppFreq)-@sin(2.0*Pi*time*LowFreq))/(Pi*time)
time = time+1.0
	dc Window*Filter
count   set count+1
 ENDIF

 IF (G3PLX==ON)
; Special minimal input crosstalk Filter
Window = p0+p1*@cos(angle)+p2*@cos(2.0*angle)+p3*@cos(3.0*angle)+p4*@cos(4.0*angle)+p5*@cos(5.0*angle)
Filter = (@sin(2.0*Pi*time*UppFreq)-@sin(2.0*Pi*time*LowFreq))/(Pi*time)
time = time+1.0
	dc Window*Filter
count   set count+1
 ENDIF

	endm


	org Y:OutFilter
time = -@cvf(OutFilterLen/2)+0.5
count   set 0
	dup OutFilterLen
angle = Pi*time/@cvf(OutFilterLen)
Window = 0.75*@cos(angle)+0.25*@cos(3*angle)
Filter = (@sin(2.0*Pi*time*OUppFrq)-@sin(2.0*Pi*time*OLowFrq))/(Pi*time)
time = time+1.0
	dc Window*Filter
count   set count+1
	endm
;-----------------------------------------------------------------------------
        org Y:TxSymbolShape

time = -0.5*@cvf(SymbolShapeLen)+0.5
count   set 0
        dup SymbolShapeLen
angle = Pi*time/@cvf(SymbolShapeLen)

 IF (BH==ON)
; Blackman Harris
Filter = h0+h1*@cos(2.0*angle)+h2*@cos(4.0*angle)
 ENDIF

 IF (COSINE==ON)
; Cosine Filter
Filter = 0.75*@cos(angle)+0.25*@cos(3.0*angle)
 ENDIF

time = time+1.0
        dc  Filter/2
count   set count+1
        endm

;-----------------------------------------------------------------------------
; Reserve Y:0100-01FF for sine/cosine lookup table
;-----------------------------------------------------------------------------
       org     y:$0100
lookup		ds	256
;-----------------------------------------------------------------------------

   end
